defmodule DecentralisedBookIndexWeb.DbiServerLive.Show do
  use DecentralisedBookIndexWeb, :live_view

  @impl true
  def render(assigns) do
    ~H"""
    <.header class="flex-col lg:flex-row">
      {@dbi_server.name}

      <:actions>
        <%= if is_nil(@dbi_server.dbi_server) do %>
          <.link patch={~p"/servers/#{@dbi_server}/edit"} phx-click={JS.push_focus()}>
            <.edit_button>
              Edit
            </.edit_button>
          </.link>
        <% end %>
      </:actions>
    </.header>


    <div>
      <dl>
        <dt class="mb-2 font-semibold leading-none text-gray-900 dark:text-white">Url</dt>
        <dd class="mb-4 font-light text-gray-500 sm:mb-5 dark:text-gray-400">{@dbi_server.url}</dd>
      </dl>
      <dl>
        <dt class="mb-2 font-semibold leading-none text-gray-900 dark:text-white">Sync on?</dt>
        <dd class="mb-4 font-light text-gray-500 sm:mb-5 dark:text-gray-400">{@dbi_server.sync_on?}</dd>
      </dl>
    </div>

    <.back navigate={~p"/servers"}>Back to servers</.back>
    """
  end

  @impl true
  def mount(_params, _session, socket) do
    {:ok, socket}
  end

  @impl true
  def handle_params(%{"id" => id}, _, socket) do
    {:noreply,
     socket
     |> assign(:page_title, page_title(socket.assigns.live_action))
     |> assign(
       :dbi_server,
       Ash.get!(DecentralisedBookIndex.Metadata.DBIServer, id, load: [:dbi_server], actor: socket.assigns.current_user)
     )}
  end

  defp page_title(:show), do: "Show server"
  defp page_title(:edit), do: "Edit server"
end
