defmodule DecentralisedBookIndexWeb.UserLive.Edit do
  use DecentralisedBookIndexWeb, :live_view

  alias DecentralisedBookIndex.Accounts.User

  @impl true
  def render(assigns) do
    ~H"""
    <.live_component
      module={DecentralisedBookIndexWeb.UserLive.FormComponent}
      id={(@user && @user.id) || :new}
      title={@page_title}
      current_user={@current_user}
      action={@live_action}
      user={@user}
    />
    """
  end

  @impl true
  def mount(_params, _session, socket) do
    {:ok,
     socket
     |> assign_new(:current_user, fn -> nil end)}
  end

  @impl true
  def handle_params(params, _url, socket) do
    socket =
      socket
      |> assign(:params, params)
      |> apply_action(socket.assigns.live_action, params)

    {:noreply, socket}
  end

  defp apply_action(socket, :edit, %{"id" => id}) do
    socket
    |> assign(:page_title, "Edit User")
    |> assign(
      :user,
      User.get_by_id!(id, actor: socket.assigns.current_user)
    )
  end

  defp apply_action(socket, :new, _params) do
    socket
    |> assign(:page_title, "New User")
    |> assign(:user, nil)
  end

  @impl true
  def handle_info({DecentralisedBookIndexWeb.UserLive.FormComponent, {:saved, _user}}, socket) do
    {:noreply, socket}
  end
end
