defmodule DecentralisedBookIndex.Sync.DataTransformers.AuthorTransformerTest do
  use ExUnit.Case, async: true

  alias DecentralisedBookIndex.Sync.DataTransformers.AuthorTransformer

  describe "correct transformations" do
    test "a json contains correct author information" do
      json_body = %{
        "data" => %{
          "attributes" => %{
            "author_alias_registry_id" => "1320a6fe-b311-45ac-bbaa-39cf29c44624",
            "avatar_url" => "/images/avatar.png",
            "dbi_server_id" => "889a323e-d104-4b5d-b276-dad5a9b1da99",
            "description" => "Something",
            "inserted_at" => "2025-03-25T19:19:15.187456Z",
            "name" => "Author",
            "updated_at" => "2025-03-26T10:16:16.124581Z"
          },
          "id" => "889a323e-d104-4b5d-b276-dad5a9b1da9d",
          "links" => %{},
          "meta" => %{},
          "relationships" => %{
            "author_alias_registry" => %{"links" => %{}, "meta" => %{}},
            "dbi_server" => %{"links" => %{}, "meta" => %{}}
          },
          "type" => "author"
        },
        "jsonapi" => %{"version" => "1.0"},
        "links" => %{
          "self" =>
            "http://localhost:4000/api/v1/json/author/889a323e-d104-4b5d-b276-dad5a9b1da9d"
        },
        "meta" => %{}
      }

      assert {:ok, author} = AuthorTransformer.from_json(json_body)

      assert %{
               id: "889a323e-d104-4b5d-b276-dad5a9b1da9d",
               name: "Author",
               description: "Something",
               avatar_url: "/images/avatar.png",
               author_alias_registry_id: "1320a6fe-b311-45ac-bbaa-39cf29c44624",
               dbi_server_id: "889a323e-d104-4b5d-b276-dad5a9b1da99",
               inserted_at: "2025-03-25T19:19:15.187456Z",
               updated_at: "2025-03-26T10:16:16.124581Z"
             } = author
    end

    test "a json doesn't contains author information \"data\" attribute" do
      json_body = %{
        "attributes" => %{
          "author_alias_registry_id" => "1320a6fe-b311-45ac-bbaa-39cf29c44624",
          "avatar_url" => "/images/avatar.png",
          "dbi_server_id" => "889a323e-d104-4b5d-b276-dad5a9b1da99",
          "description" => "Something",
          "inserted_at" => "2025-03-25T19:19:15.187456Z",
          "name" => "Author",
          "updated_at" => "2025-03-26T10:16:16.124581Z"
        },
        "id" => "889a323e-d104-4b5d-b276-dad5a9b1da9d",
        "links" => %{},
        "meta" => %{},
        "relationships" => %{
          "author_alias_registry" => %{"links" => %{}, "meta" => %{}},
          "dbi_server" => %{"links" => %{}, "meta" => %{}}
        },
        "type" => "author"
      }

      assert {:ok, author} = AuthorTransformer.from_json(json_body)

      assert %{
               id: "889a323e-d104-4b5d-b276-dad5a9b1da9d",
               name: "Author",
               description: "Something",
               avatar_url: "/images/avatar.png",
               author_alias_registry_id: "1320a6fe-b311-45ac-bbaa-39cf29c44624",
               dbi_server_id: "889a323e-d104-4b5d-b276-dad5a9b1da99",
               inserted_at: "2025-03-25T19:19:15.187456Z",
               updated_at: "2025-03-26T10:16:16.124581Z"
             } = author
    end
  end
end
