defmodule DecentralisedBookIndex.Sync.DataTransformers.DBIServerSyncTest do
  use DecentralisedBookIndex.DataCase, async: true

  alias DecentralisedBookIndex.Sync.DBIServerSync
  alias DecentralisedBookIndex.Metadata

  alias DecentralisedBookIndex.TestEndpoints
  @test_server_endpoint TestEndpoints.test_api_endpoint()

  setup do
    user = generate(user(role: :admin))
    %{user: user}
  end

  describe "sync dbi_server transformations" do
    test "a new dbi_server will be created" do
      server = generate(dbi_server(url: @test_server_endpoint))

      dbi_server = %{
               id: "0c0647ec-07ef-4caa-b683-5847dbfbe5cc",
               name: "Test",
               url: "http://localhost:4001",
               inserted_at: "2025-03-22T20:07:30.766249Z",
               updated_at: "2025-05-04T18:48:44.213309Z"
      }

      {:ok, inserted_at, 0} = DateTime.from_iso8601(dbi_server[:inserted_at])
      {:ok, updated_at, 0} = DateTime.from_iso8601(dbi_server[:updated_at])

      assert :ok = DBIServerSync.create_update(dbi_server, server.id)
      assert {:ok, saved_dbi_server} = Metadata.get_dbi_server_by_id(dbi_server.id)

      dbi_server =
        dbi_server
        |> Map.replace(:inserted_at, inserted_at)
        |> Map.replace(:updated_at, updated_at)

      assert dbi_server = saved_dbi_server
      assert server.id == saved_dbi_server.dbi_server_id
    end

    test "update an existing dbi_server", %{user: user} do
      server = generate(dbi_server(url: @test_server_endpoint))

      {:ok, dbi_server} = Metadata.create_dbi_server("Test", "http://localhost:4001", false, actor: user)

      dbi_server_attrs = %{
               id: dbi_server.id,
               name: "Test",
               url: "http://localhost:4001",
               inserted_at: "2025-03-22T20:07:30.766249Z",
               updated_at: "2025-05-04T18:48:44.213309Z"
      }

      assert :ok = DBIServerSync.create_update(dbi_server_attrs, server.id)
      assert {:ok, saved_dbi_server} = Metadata.get_dbi_server_by_id(dbi_server.id)

      assert dbi_server_attrs = saved_dbi_server
      assert server.id == saved_dbi_server.dbi_server_id
    end
  end
end
